@@ -71,22 +71,6 @@ renv_update_find_git_impl <- function(record) {
 
 renv_update_find_github <- function(records) {
 
-  # check for GITHUB_PAT
-  if (!renv_envvar_exists("GITHUB_PAT")) {
-
-    msg <- paste(
-      "GITHUB_PAT is unset. Updates may fail due to GitHub's API rate limit.",
-      "",
-      "To increase your GitHub API rate limit:",
-      "- Use `usethis::create_github_token()` to create a Personal Access Token (PAT).",
-      "- Use `usethis::edit_r_environ()` and add the token as `GITHUB_PAT`.",
-      sep = "\n"
-    )
-
-    warning(msg, call. = FALSE)
-
-  }
-
   names(records) <- map_chr(records, `[[`, "Package")
   results <- renv_parallel_exec(records, function(record) {
     catch(renv_update_find_github_impl(record))
@@ -246,6 +230,9 @@ renv_update_find <- function(records) {
 #'   Use `renv::update(exclude = <...>)` to update all packages except for
 #'   a specific set of excluded packages.
 #'
+#' @param lock Boolean; update the `renv.lock` lockfile after the successful
+#'   installation of the requested packages?
+#'
 #' @return A named list of package records which were installed by renv.
 #'
 #' @export
@@ -261,9 +248,11 @@ update <- function(packages = NULL,
                    ...,
                    exclude = NULL,
                    library = NULL,
+                   type    = NULL,
                    rebuild = FALSE,
                    check   = FALSE,
                    prompt  = interactive(),
+                   lock    = FALSE,
                    project = NULL)
 {
   renv_consent_check()
@@ -279,6 +268,12 @@ update <- function(packages = NULL,
   library <- nth(libpaths, 1L)
   renv_scope_libpaths(libpaths)
 
+  # check for explicitly-provided type -- we handle this specially for PPM
+  if (!is.null(type)) {
+    renv_scope_binding(the, "install_pkg_type", type)
+    renv_scope_options(pkgType = type)
+  }
+
   # resolve exclusions
   exclude <- c(exclude, settings$ignored.packages(project = project))
 
@@ -286,7 +281,16 @@ update <- function(packages = NULL,
   if (config$pak.enabled() && !recursing()) {
     packages <- setdiff(packages, exclude)
     renv_pak_init()
-    return(renv_pak_install(packages, libpaths, project))
+    return(
+      renv_pak_install(
+        packages = packages,
+        library  = libpaths,
+        rebuild  = rebuild,
+        type     = type,
+        prompt   = prompt,
+        project  = project
+      )
+    )
   }
 
   # get package records
@@ -407,6 +411,7 @@ update <- function(packages = NULL,
     library  = libpaths,
     rebuild  = rebuild,
     prompt   = prompt,
+    lock     = lock,
     project  = project
   )
 
