@@ -397,13 +397,11 @@ test_that("packages installed from CRAN using pak are handled", {
   library <- renv_paths_library()
   ensure_directory(library)
   pak <- renv_namespace_load("pak")
-  suppressMessages(pak$pkg_install("toast"))
+  quietly(pak$pkg_install("toast"))
   record <- renv_snapshot_description(package = "toast")
 
-  expect_named(
-    record,
-    c("Package", "Version", "Source", "Repository", "Requirements", "Hash")
-  )
+  expected <- c("Package", "Version", "Source", "Repository")
+  expect_contains(names(record), expected)
 
   expect_identical(record$Source, "Repository")
   expect_identical(record$Repository, "CRAN")
@@ -413,6 +411,7 @@ test_that("packages installed from CRAN using pak are handled", {
 test_that("packages installed from Bioconductor using pak are handled", {
   skip_on_cran()
   skip_if_not_installed("pak")
+  skip_if(devel())
 
   renv_tests_scope()
   library <- renv_paths_library()
@@ -614,3 +613,54 @@ test_that("standard remotes drop RemoteSha if it's a version", {
   expect_null(record[["RemoteSha"]])
 
 })
+
+test_that("a package's hash can be re-generated from lockfile", {
+  
+  project <- renv_tests_scope("breakfast")
+  init()
+  
+  lockfile <- snapshot(lockfile = NULL)
+  records <- renv_lockfile_records(lockfile)
+  
+  enumerate(records, function(package, record) {
+    path <- system.file("DESCRIPTION", package = package)
+    actual <- renv_hash_description(path)
+    expected <- renv_hash_record(record)
+    expect_equal(actual, expected)
+  })
+  
+})
+
+test_that("lockfiles are stable (v1)", {
+  
+  renv_scope_options(renv.lockfile.version = 1L)
+  
+  project <- renv_tests_scope("breakfast")
+  init()
+  
+  expect_snapshot(. <- writeLines(readLines("renv.lock")))
+  
+})
+
+test_that("lockfiles are stable (v2)", {
+  
+  renv_scope_options(renv.lockfile.version = 2L)
+  
+  project <- renv_tests_scope("breakfast")
+  init()
+
+  expect_snapshot(. <- writeLines(readLines("renv.lock")))
+
+})
+
+# # https://github.com/rstudio/renv/issues/2073
+test_that("empty .ipynb files are handled gracefully", {
+
+  skip_on_cran()
+  project <- renv_tests_scope("bread")
+  init()
+
+  writeLines("", con = "example.ipynb")
+  snapshot()
+
+})
